#pragma once

#ifndef __BASE_H__
#define __BASE_H__

#include <sys/ksys.h>
#include <string.h>
#include <stdlib.h>

#include <string>
#include <fstream>

#include "small.hpp"
#include "filesystem.hpp"
#include "mouse.hpp"

/// \brief Основное пространство имён
/// \author Egor00f
namespace KolibriLib
{
     
    /// @brief Работа с системой
    namespace OS
    {

        /// \brief Таблица стандартных(системных) цветов
        ksys_colors_table_t sys_color_table;

        /// @brief Получить системные цвета
        /// @paragraph Функция изменяет переменную @link sys_color_table
        /// @return Таблица системных цветов
        ksys_colors_table_t GetSystemColors();

        /// \brief Ждать ивента
        /// \return Ивент
        inline unsigned int WaitEvent();

        /// \brief Ждать ивента
        /// \param Таймаут (в 1/100 секунды)
        /// \return Ивент
        inline unsigned int WaitEvent(uint32_t TimeOut)
        {
            _ksys_wait_event_timeout(TimeOut);
        }

        /// \brief Проверить пришёл ли ли ивент
        /// \return Ивен
        inline unsigned int CheckEvent()
        {
            return _ksys_check_event();
        }

        /// \brief Запустить программу
        /// \param AppName путь до программы + имя
        /// \param args аргументы
        /// \return то что возвращает сама программа, -1 если исполняемы файл не найден
        inline int Exec(std::string AppName, std::string args)
        {
            if(filesystem::Exist(AppName))//Проверка на существование
            {
                char *a;
                strcat(a, args.c_str());
                return _ksys_exec(AppName.c_str(), a);
            }
            else
            {
                return -1;
            }
        }

        /// @brief Получить системное время
        /// @return 
        inline ksys_time_t GetTime()
        {
            return _ksys_get_time();
        }


    }

    /// \brief Подождать
    /// \param time время задержки(в 1/100 секунды)
    inline void Wait(unsigned time)
    {
        _ksys_delay(time);
    }

    /// @brief Инициализация
    /// @paragraph Просто не заморачиваяся, и вызывай в начале программы
    void init()
    {
        _ksys_set_event_mask(0x07);
        OS::GetSystemColors();

        if(filesystem::Exist("/kolibrios/etc"))
        {
            filesystem::mkdir("/kolibrios/etc");
        }

        char *a;
        strcat(a, ConfigurationDir.c_str());
        strcat(a, "cursors");
        std::ofstream file(a);

        if(file.is_open())
        {
            std::string buff;
            file << buff;
            if(buff == "cursors loaded!")
            {

            }
            else
            {
                mouse::pointer = mouse::LoadCursor(CursorsDir + "pointer.cur");
                mouse::dgn1    = mouse::LoadCursor(CursorsDir + "dgn1.cur");
                mouse::dgn2    = mouse::LoadCursor(CursorsDir + "dgn2.cur");
                mouse::link    = mouse::LoadCursor(CursorsDir + "link.cur");
                mouse::horz    = mouse::LoadCursor(CursorsDir + "horz.cur");
                mouse::vert    = mouse::LoadCursor(CursorsDir + "vert.cur");
                mouse::beam    = mouse::LoadCursor(CursorsDir + "beam.cur");
                mouse::busy    = mouse::LoadCursor(CursorsDir + "busy.cur");

            }
        }
        else
        {
            std::ofstream file();
        }
        file.close();
    }

    /// @brief Проверить какая клавиша клавиатуры нажата
    /// @return код(char) нажатой клавишы клавиатуры
    char CheckKeyboard()
    {
        return _ksys_get_key().code;
    }
}



#endif // __BASE_H__