#ifndef __GRAPHIC_H__
#define __GRAPHIC_H__


#include <sys/ksys.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <math.h>

#include <fstream>

#include "base.hpp"
#include "filesystem.hpp"

namespace KolibriLib
{
    // Графика
    namespace graphic
    {
        /// @brief Нарисовать линию
        /// @param a точка первая
        /// @param b точка вторая
        inline void DrawLine(const point& a, const point& b, const ksys_color_t color = OS::sys_color_table.work_graph)
        {
            _ksys_draw_line(a.x, a.y, b.x, b.y, color);
        }

        /// @brief Закрасить пиксель точку
        /// @param position координаты
        /// @param color Цвет
        inline void DrawPixel(const point& position, const ksys_color_t color = OS::sys_color_table.work_text)
        {
            _ksys_draw_pixel(position.x, position.y, color);
        }

        /// @brief Нарисовать окружность
        /// @param coord Координаты центра
        /// @param Radius радиус окружности
        /// @param detalization Детализация прорисовки окружности
        /// @paragraph Круг рисуется с помошью линий
        /// @param color Цвет
        void DrawCircle(point coord, unsigned Radius, unsigned detalization = 36, ksys_color_t color = OS::sys_color_table.work_graph)
        {
            point buff;
            unsigned b = Radius;
            unsigned c = 0;
            for (unsigned angle = 1; angle <= detalization * 10; angle += (36 / detalization))
            {
                buff = {coord.x + b, coord.y + c};
                b = Radius * cos(angle);
                c = Radius * sin(angle);
                point n = {coord.x + b, coord.y + c};
                DrawLine(buff, n);
            }
        }

        /// @brief Нарисовать закрашенный прямоугольник
        /// @param position позиция левого верхнего угла
        /// @param size Размеры
        /// @param color Цвет
        void DrawRectangleFill(point position, const point& size, const ksys_color_t color = OS::sys_color_table.work_graph)
        {
            __asm__ __volatile__(
                "int $0x40" 
                ::"a"(13), "b"((position.x << 16) + size.x), "c"((position.y << 16) + size.y), "d"(color));
        }

        /// @brief Нарисовать круг(закрашенный)
        /// @param coord Координаты центра
        /// @param Radius Радиус круга
        /// @param color Цвет
        void DrawCircleFill(point coord, unsigned Radius, ksys_color_t color = OS::sys_color_table.work_graph);

        /// @brief Нарисовать точку
        /// @param position Координаты
        /// @param size Радиус точки в рx
        /// @param color Цвет
        /// @param fill Закрашенная/Выколотая точка
        /// @paragraph Функция рисует закрашенную точку если fill = true
        void DrawPoint(const point& position, const unsigned& Radius, const ksys_color_t& color = OS::sys_color_table.work_graph, bool fill = false);


        /// @brief Нарисовать прямоугольник
        /// @param a Точка в левом верхнем углу
        /// @param b Точка в правом нижнем углу
        void DrawRectangleLines(point a, point b, ksys_color_t color = OS::sys_color_table.work_graph);

        

        /// @brief Нарисовать треугольник
        /// @param a первая точка
        /// @param b вторая точка
        /// @param c третья точка
        /// @param color цвет линий
        inline void DrawTriangle(point a, point b, point c, ksys_color_t color = OS::sys_color_table.work_graph);

        //=============================================================================================================================================================

        
        //=============================================================================================================================================================

        
    }
} // namespace KolibriLib


#endif // __GRAPHIC_H__