#ifndef __IMAGE_H__
#define __IMAGE_H__

#include <sys/ksys.h>
#include <string>
#include <fstream>

#include "base.hpp"
#include "filesystem.hpp"

namespace KolibriLib
{
    namespace Image
    {
        template<class ImageBits>
        struct ARGB
        {
            ImageBits alpha;
            ImageBits red;
            ImageBits green;
            ImageBits blue;
        };

        template <class ImageBits>
        /// @brief Класс для работы с изображениями
        class img
        {
        private:
            ARGB<ImageBits> *_d;
            point _size;
        public:
            img(std::string Path);
            ~img();

            /// @brief Загрузить изображение из файла
            /// @param Path Путь до файла с изображением
            /// @return true если удачно, false если неудачно
            bool LoadImage(std::string Path);
            

            /// @briefВывести изображение
            /// @param coord координаты
            /// @param size
            /// @warning Функция пока что очень медленная, может быть когданибудь позже от=птимизирую...
            void DrawImage(point coord = {0, 0}, point size = {0, 0});

            void LoadFromArray();

            /// @brief Получить размеры изображения
            /// @return @link _size
            point GetSize();

            ARGB<ImageBits>* GetImagePtr();
        };

        img<class ImageBits>::img(std::string Path)
        {
            _size = {0, 0};
            LoadImage(Path);
            _d = (ARGB<ImageBits> *)malloc(sizeof(ARGB<ImageBits>));
        }

        img<class ImageBits>::~img()
        {
            free(_d);
        }

        bool img<class ImageBits>::LoadImage(std::string Path)
        {
            bool extcode;
            if (filesystem::Exist(Path)) // Если файл с изображением существует
            {
                std::string Type;
                std::ifstream img;

                {
                    unsigned i = Path.length();

                    while (Path[i] != '.')
                    {
                        Type.push_back(Path[i]);
                        i--;
                    }
                }

                if (Type == "raw")
                {
                    rgb_t buff;
                    while(!img.eof())
                    {
                        img.read((char*)&_d, sizeof(rgb_t));
                    }
                    
                }
                else if (Type == "jpeg")
                {
                    /* code */
                }
                else
                {
                    extcode = false;
                }

                img.close();
            }
            else
            {
                return extcode = false;
            }
            return extcode;
        }

        void img<class ImageBits>::DrawImage(point coord, point size)
        {
            if (size.x == 0 && size.y == 0)
            {
                size = _size;
            }

            rgb_t *buff = (rgb_t*)malloc( _size.x * _size.y * sizeof(rgb_t));

            for (unsigned i = 0; i < _size.x * _size.y; i++)
            {
                buff[i].red     = _d[i].red;
                buff[i].green   = _d[i].green;
                buff[i].blue    = _d[i].blue;
            }

            _ksys_draw_bitmap(buff, coord.x, coord.y, size.x, size.y);
        }
        
        point img<class ImageBits>::GetSize()
        {
            return _size;
        }
        
        ARGB<ImageBits>* img<ImageBits>::GetImagePtr()
        {
            return _d;
        }

    

    } // namespace Image
    
}


#endif // __IMAGE_H__